#!/bin/bash

# Example:  ControlScript.sh CompareImageSingle examples lennon-1.jpg examples lennon-2.jpg out len1-len2-compareSingle.txt
#           ControlScript.sh CompareImageMulti examples_len examples_lon out len1-len2-compareMulti.txt
#           ControlScript.sh CreateClassifier persons tmp personsOut personsNewClassifier
#           ControlScript.sh Classify persons/lennon personsOut personsNewClassifier.pkl out classifyOut.txt
#

SharedFolder="/SharedFolder"
BasePath="/root/openface"
ComparePath="demos"
CompareBin="compare.py"
CompareImageSingleName="CompareImageSingle"
CompareImageMultiName="CompareImageMulti"
CreateClassifier="CreateClassifier"
CreateClassifierPath="util"
CreateClassifierAlignBin="align-dlib.py"
CreateClassifierAlignParallels="8"
CreateClassifierAlignParam1="align outerEyesAndNose"
CreateClassifierAlignParam2="--size 96"
CreateRepresentationsPath="batch-represent"
CreateRepresentationsBin="main.lua"
CreateClassificatioModelPath="demos"
CreateClassificatioModelBin="classifier.py"
Classify="Classify"

OperationMode="${1}"

if [ ${#} -lt 1 ]; then
  echo "Use: ${0} <OperationMode> <<ModeDependendSettings>>"
  exit 1
fi

case ${OperationMode} in
  ${CompareImageSingleName})
    echo "OperationMode: ${CompareImageSingleName}"
    if [ ${#} -ne 7 ]; then
      echo "Use: ${0} ${CompareImageSingleName} <InFolder1> <InFile1> <InFolder2> <InFile2> <OutFolder> <OutFile>"
      exit 1
    fi
    InFolder1="${2}"
    InFile1="${3}"
    InFolder2="${4}"
    InFile2="${5}"
    OutFolder="${6}"
    OutFile="${7}"
    if [ ! -d ${SharedFolder}/${InFolder1} ]; then
      echo "Error: InFolder1 not exists"
      exit 1
    fi
    if [ ! -f ${SharedFolder}/${InFolder1}/${InFile1} ]; then
      echo "Error: InFile1 not exists"
      exit 1
    fi
    if [ ! -d ${SharedFolder}/${InFolder2} ]; then
      echo "Error: InFolder2 not exists"
      exit 1
    fi
    if [ ! -f ${SharedFolder}/${InFolder2}/${InFile2} ]; then
      echo "Error: InFile2 not exists"
      exit 1
    fi
    if [ -d ${SharedFolder}/${OutFolder} ]; then
      echo "Warning: OutFolder exists"
    else
      mkdir ${SharedFolder}/${OutFolder}
    fi
    if [ -f ${SharedFolder}/${OutFolder}/${OutFile} ]; then
      echo "Warning: OutFile exists"
    fi
    ${BasePath}/${ComparePath}/${CompareBin} ${SharedFolder}/${InFolder1}/${InFile1} ${SharedFolder}/${InFolder2}/${InFile2} > ${SharedFolder}/${OutFolder}/${OutFile}
    ;;

  ${CompareImageMultiName})
    echo "OperationMode: ${CompareImageMultiName}"
    if [ ${#} -ne 5 ]; then
      echo "Use: ${0} ${CompareImageMultiName} <InFolder1> <InFolder2> <OutFolder> <OutFile>"
      exit 1
    fi
    InFolder1="${2}"
    InFolder2="${3}"
    OutFolder="${4}"
    OutFile="${5}"
    if [ ! -d ${SharedFolder}/${InFolder1} ]; then
      echo "Error: InFolder1 not exists"
      exit 1
    fi
    if [ ! -d ${SharedFolder}/${InFolder2} ]; then
      echo "Error: InFolder2 not exists"
      exit 1
    fi
    if [ -d ${SharedFolder}/${OutFolder} ]; then
      echo "Warning: OutFolder exists"
    else
      mkdir ${SharedFolder}/${OutFolder}
    fi
    if [ -f ${SharedFolder}/${OutFolder}/${OutFile} ]; then
      echo "Warning: OutFile exists"
    fi
    ${BasePath}/${ComparePath}/${CompareBin} ${SharedFolder}/${InFolder1}/* ${SharedFolder}/${InFolder2}/* | grep -ve "with ${SharedFolder}/${InFolder1}/"  > ${SharedFolder}/${OutFolder}/${OutFile}
    ;;

  ${CreateClassifier})
    echo "OperationMode: ${CreateClassifier}"
    if [ ${#} -ne 5 ]; then
      echo "Use: ${0} ${CreateClassifier} <InFolder> <TmpFolder> <OutFolderClassifier> <OutFileClassifier>"
      exit 1
    fi
    InFolder="${2}"
    TmpFolder="${3}"
    OutFolderClassifier="${4}"
    OutFileClassifier="${5}"
    if [ ! -d ${SharedFolder}/${InFolder} ]; then
      echo "Error: InFolder not exists"
      exit 1
    fi
    if [ -d ${SharedFolder}/${TmpFolder} ]; then
      echo "Warning: TmpFolder exists"
    else
      mkdir ${SharedFolder}/${TmpFolder}
    fi
    if [ -d ${SharedFolder}/${OutFolderClassifier} ]; then
      echo "Warning: OutFolderClassifier exists"
    else
      mkdir ${SharedFolder}/${OutFolderClassifier}
    fi
    if [ -f ${SharedFolder}/${OutFolderClassifier}/${OutFileClassifier} ]; then
      echo "Warning: OutFileClassifier exists"
    fi

    # Align Images
    for i in $(seq ${CreateClassifierAlignParallels}); do
      ${BasePath}/${CreateClassifierPath}/${CreateClassifierAlignBin} ${SharedFolder}/${InFolder}/ ${CreateClassifierAlignParam1} ${SharedFolder}/${TmpFolder}/AlignedImages/ ${CreateClassifierAlignParam2} & 
    done
    wait # to finish Align

    # Create Representations
    ${BasePath}/${CreateRepresentationsPath}/${CreateRepresentationsBin} -data ${SharedFolder}/${TmpFolder}/AlignedImages/ -outDir ${SharedFolder}/${TmpFolder}/Features/

    # Create Classification Model
    ${BasePath}/${CreateClassificatioModelPath}/${CreateClassificatioModelBin} --cuda train ${SharedFolder}/${TmpFolder}/Features/
    mv ${SharedFolder}/${TmpFolder}/Features/classifier.pkl ${SharedFolder}/${OutFolderClassifier}/${OutFileClassifier}.pkl
    ;;

  ${Classify})
    echo "OperationMode: ${Classify}"
    if [ ${#} -ne 6 ]; then
      echo "Use: ${0} ${Classify} <InFolder> <InClassifierFolder> <InClassifierFile> <OutFolder> <OutFile>"
      exit 1
    fi
    InFolder="${2}"
    InClassifierFolder="${3}"
    InClassifierFile="${4}"
    OutFolder="${5}"
    OutFile="${6}"
    if [ ! -d ${SharedFolder}/${InFolder} ]; then
      echo "Error: InFolder not exists"
      exit 1
    fi
    if [ ! -d ${SharedFolder}/${InClassifierFolder} ]; then
      echo "Error: InClassifierFolder not exists"
      exit 1
    fi
    if [ ! -f ${SharedFolder}/${InClassifierFolder}/${InClassifierFile} ]; then
      echo "Error: InClassifierFile not exists"
      exit 1
    fi
    if [ -d ${SharedFolder}/${OutFolder} ]; then
      echo "Warning: OutFolder exists"
    else
      mkdir ${SharedFolder}/${OutFolder}
    fi
    if [ -f ${SharedFolder}/${OutFolder}/${OutFile} ]; then
      echo "Warning: OutFile exists"
    fi

    # Classify images from InFolder
    ${BasePath}/${CreateClassificatioModelPath}/${CreateClassificatioModelBin} infer ${SharedFolder}/${InClassifierFolder}/${InClassifierFile} ${SharedFolder}/${InFolder}/* 2> ${SharedFolder}/${OutFolder}/${OutFile}.errorlog > ${SharedFolder}/${OutFolder}/${OutFile}
    ;;

  *)
    echo "Error: Unknown OperationMode"
    exit 1
    ;;
esac

